﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Strecken
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        List<Polyline> lines = new List<Polyline>();

        // Geht nur, wenn Canvas einen Background hat, ggf. Transparent
        private void Canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            // Damit wir z.B. mitbekommen, wenn die Maus außerhalb der Zeichenfläche losgelassen wird.
            Mouse.Capture(myCanvas);

            Polyline p = new Polyline();
            p.Stroke = Brushes.Blue;
            p.Points.Add(e.GetPosition(myCanvas));
            myCanvas.Children.Add(p);
            lines.Add(p);
        }

        private void Canvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed && e.Source == myCanvas)
            {
                lines.Last().Points.Add(e.GetPosition(myCanvas));
            }
        }

        List<Intersection> intersections = new List<Intersection>();

        private void Canvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Mouse.Capture(null);

            foreach (var item in intersections)
	        {
		        myCanvas.Children.Remove(item.Disk);
	        }
            intersections.Clear();

            for (int i = 0; i < lines.Count; i++)
            {
                for (int j = i; j < lines.Count; j++)
                {
                    for (int n = 0; n < lines[i].Points.Count - 1; n++)
                    {
                        Point a = lines[i].Points[n];
                        Point b = lines[i].Points[n + 1];

                        for (int m = 0; m < lines[j].Points.Count - 1; m++)
                        {
                            Point c = lines[j].Points[m];
                            Point d = lines[j].Points[m + 1];

                            double denominator = (b.X - a.X) * (d.Y - c.Y) - (b.Y - a.Y) * (d.X - c.X);
                            double lambda = ((c.X - a.X) * (d.Y - c.Y) - (c.Y - a.Y) * (d.X - c.X)) / denominator;
                            if (lambda >= 0.0 && lambda < 1.0) // Randnotiz: lambda könnte auch +INF oder -INF werden!
                            {
                                double mu = - ((b.X - a.X) * (c.Y - a.Y) - (b.Y - a.Y) * (c.X - a.X)) / denominator;
                                if (mu >= 0.0 && mu < 1.0) // Randnotiz: mu könnte auch +INF oder -INF werden!
                                {
                                    Ellipse ell = new Ellipse();
                                    ell.Fill = Brushes.Blue;
                                    ell.Width = 6.0;
                                    ell.Height = 6.0;
                                    ell.MouseEnter += showNeighbors;

                                    myCanvas.Children.Add(ell);

                                    double x = a.X + lambda * (b.X - a.X);
                                    double y = a.Y + lambda * (b.Y - a.Y);
                                    Canvas.SetLeft(ell, x - 0.5 * ell.Width);
                                    Canvas.SetTop(ell, y - 0.5 * ell.Height);

                                    intersections.Add(new Intersection(ell, lines[i], n, lines[j], m, new Point(x, y)));
                                }
                            }
                        }
                    }
                }
            }
        }

        List<Intersection> neighbors = new List<Intersection>();

        void showNeighbors(object sender, MouseEventArgs e)
        {
            Ellipse thisEllipse = (Ellipse)sender;

            Intersection myIntersection = intersections.Find(x => thisEllipse == x.Disk); // oder mit for-Schleife oder mit foreach-Schleife

            foreach (var item in neighbors)
            {
                item.Disk.Fill = Brushes.Blue;
            }

            neighbors.Clear();

            // Finde alle anderen Intersections, die auch auf myIntersection.PolylineA liegen, und zwar vorher;
            // nehme davon die nächstliegende (die nach dem Sortieren letzte) oder null, falls es keine gibt.
            // Das ist hier sehr modern geschrieben und geht natürlich auch mit for-Schleifen oder mit foreach-Schleifen.
            Intersection i = intersections
                .FindAll(x => x != myIntersection &&
                    (x.PolylineA == myIntersection.PolylineA && x.IndexA < myIntersection.IndexA
                     || x.PolylineB == myIntersection.PolylineA && x.IndexB < myIntersection.IndexA))
                .OrderBy(x => (x.PolylineA == myIntersection.PolylineA) ? x.IndexA : x.IndexB)
                .LastOrDefault();
            if (i != null)
            {
                neighbors.Add(i);
            }

            i = intersections
                .FindAll(x => x != myIntersection &&
                    (x.PolylineA == myIntersection.PolylineA && x.IndexA > myIntersection.IndexA
                     || x.PolylineB == myIntersection.PolylineA && x.IndexB > myIntersection.IndexA))
                .OrderBy(x => (x.PolylineA == myIntersection.PolylineA) ? x.IndexA : x.IndexB)
                .FirstOrDefault();
            if (i != null && !neighbors.Contains(i))
            {
                neighbors.Add(i);
            }

            i = intersections
                .FindAll(x => x != myIntersection &&
                    (x.PolylineA == myIntersection.PolylineB && x.IndexA < myIntersection.IndexB
                        || x.PolylineB == myIntersection.PolylineB && x.IndexB < myIntersection.IndexB))
                .OrderBy(x => (x.PolylineA == myIntersection.PolylineB) ? x.IndexA : x.IndexB)
                .LastOrDefault();
            if (i != null && !neighbors.Contains(i))
            {
                neighbors.Add(i);
            }

            i = intersections
                .FindAll(x => x != myIntersection &&
                    (x.PolylineA == myIntersection.PolylineB && x.IndexA > myIntersection.IndexB
                        || x.PolylineB == myIntersection.PolylineB && x.IndexB > myIntersection.IndexB))
                .OrderBy(x => (x.PolylineA == myIntersection.PolylineB) ? x.IndexA : x.IndexB)
                .FirstOrDefault();
            if (i != null && !neighbors.Contains(i))
            {
                neighbors.Add(i);
            }

            foreach (var item in neighbors)
            {
                item.Disk.Fill = Brushes.Red;
            }
        }
    }

    class Intersection
    {
        public Ellipse Disk { private set; get; }
        public Polyline PolylineA { private set; get; }
        public int IndexA { private set; get; }
        public Polyline PolylineB { private set; get; }
        public int IndexB { private set; get; }
        public Point Location { private set; get; }

        public Intersection(Ellipse disk, Polyline polylineA, int indexA, Polyline polylineB, int indexB, Point location)
        {
            Disk = disk;
            PolylineA = polylineA;
            IndexA = indexA;
            PolylineB = polylineB;
            IndexB = indexB;
            Location = location;
        }
    }
}
